/**
 * This file exports all GIR module type definitions.
 */

import './glib-2.0.d.ts';
import './gio-2.0.d.ts';
import './gobject-2.0.d.ts';
import './xrandr-1.3.d.ts';
import './xlib-2.0.d.ts';
import './xft-2.0.d.ts';
import './xfixes-4.0.d.ts';
import './win32-1.0.d.ts';
import './libxml2-2.0.d.ts';
import './freetype2-2.0.d.ts';
import './fontconfig-2.0.d.ts';
import './cairo-1.0.d.ts';
import './xmlb-2.0.d.ts';
import './webkitwebprocessextension-6.0.d.ts';
import './webkit2webextension-4.1.d.ts';
import './webkit2-4.1.d.ts';
import './webkit-6.0.d.ts';
import './vulkan-1.0.d.ts';
import './tsparql-3.0.d.ts';
import './tracker-3.0.d.ts';
import './soup-3.0.d.ts';
import './secret-1.d.ts';
import './rsvg-2.0.d.ts';
import './polkit-1.0.d.ts';
import './pangoot-1.0.d.ts';
import './pangofc-1.0.d.ts';
import './pangoft2-1.0.d.ts';
import './pangocairo-1.0.d.ts';
import './pango-1.0.d.ts';
import './notify-0.7.d.ts';
import './nice-0.1.d.ts';
import './manette-0.2.d.ts';
import './libproxy-1.0.d.ts';
import './json-1.0.d.ts';
import './javascriptcore-6.0.d.ts';
import './javascriptcore-4.1.d.ts';
import './ibus-1.0.d.ts';
import './harfbuzz-0.0.d.ts';
import './handy-1.d.ts';
import './gtksource-5.d.ts';
import './gtk-4.0.d.ts';
import './gtk-3.0.d.ts';
import './gstwebrtc-1.0.d.ts';
import './gstvulkanxcb-1.0.d.ts';
import './gstvulkanwayland-1.0.d.ts';
import './gstvulkan-1.0.d.ts';
import './gstvideo-1.0.d.ts';
import './gstva-1.0.d.ts';
import './gsttranscoder-1.0.d.ts';
import './gsttag-1.0.d.ts';
import './gstsdp-1.0.d.ts';
import './gstrtsp-1.0.d.ts';
import './gstrtp-1.0.d.ts';
import './gstplayer-1.0.d.ts';
import './gstplay-1.0.d.ts';
import './gstpbutils-1.0.d.ts';
import './gstnet-1.0.d.ts';
import './gstmse-1.0.d.ts';
import './gstmpegts-1.0.d.ts';
import './gstinsertbin-1.0.d.ts';
import './gstglx11-1.0.d.ts';
import './gstglwayland-1.0.d.ts';
import './gstglegl-1.0.d.ts';
import './gstgl-1.0.d.ts';
import './gstdxva-1.0.d.ts';
import './gstcuda-1.0.d.ts';
import './gstcontroller-1.0.d.ts';
import './gstcodecs-1.0.d.ts';
import './gstcheck-1.0.d.ts';
import './gstbase-1.0.d.ts';
import './gstbadaudio-1.0.d.ts';
import './gstaudio-1.0.d.ts';
import './gstapp-1.0.d.ts';
import './gstanalytics-1.0.d.ts';
import './gstallocators-1.0.d.ts';
import './gst-1.0.d.ts';
import './gsk-4.0.d.ts';
import './graphene-1.0.d.ts';
import './giounix-2.0.d.ts';
import './geoclue-2.0.d.ts';
import './gee-0.8.d.ts';
import './gdkx11-4.0.d.ts';
import './gdkx11-3.0.d.ts';
import './gdkwayland-4.0.d.ts';
import './gdkpixdata-2.0.d.ts';
import './gdkpixbuf-2.0.d.ts';
import './gdk-4.0.d.ts';
import './gdk-3.0.d.ts';
import './gcrui-3.d.ts';
import './gcr-4.d.ts';
import './gcr-3.d.ts';
import './gck-2.d.ts';
import './gck-1.d.ts';
import './gudev-1.0.d.ts';
import './gmodule-2.0.d.ts';
import './glibunix-2.0.d.ts';
import './gl-1.0.d.ts';
import './girepository-3.0.d.ts';
import './girepository-2.0.d.ts';
import './gdesktopenums-3.0.d.ts';
import './gcab-1.0.d.ts';
import './dex-1.d.ts';
import './dbusglib-1.0.d.ts';
import './dbus-1.0.d.ts';
import './cudagst-1.0.d.ts';
import './atspi-2.0.d.ts';
import './atk-1.0.d.ts';
import './appstreamcompose-1.0.d.ts';
import './appstream-1.0.d.ts';
import './adw-1.d.ts';
